/*
 * menu.c
 *
 * Created: 20.09.2017 12:36:58
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <avr/pgmspace.h>
#include "defs.h"
#include "buttons.h"
#include "menu.h"
#include "menu_defs.h"
#include "display.h"
#include "settings.h"
#include "dstemp.h"


#define MENU_UP_BUTTON					BUTTON2_CLICK
#define MENU_DN_BUTTON					BUTTON3_CLICK
#define MENU_ESC_BUTTON					BUTTON1_CLICK
#define MENU_ENTER_BUTTON				BUTTON4_CLICK
#define MENU_REPEAT_BUTTON				BUTTON_REPEAT

#define MENU_EDIT_DISABLE				0
#define MENU_EDIT_ENABLE				1
#define MENU_EDIT_CHANGE				2
#define MENU_EDIT_CHARACTER1			">"
#define MENU_EDIT_CHARACTER2			"<"

#define MENU_MAIN_PAGE_ALL				0
#define MENU_MAIN_PAGE_T1				1
#define MENU_MAIN_PAGE_T2				2
#define MENU_MAIN_PAGE_TR				3

#define MENU_MAIN_PAGE_PROFILE			4

#define MENU_MAIN_PAGE_MAX1				5
#define MENU_MAIN_PAGE_MAX2				6
//#define MENU_MAIN_PAGE_MAXR				6

#define MENU_MAIN_PAGE_OUT1				7
#define MENU_MAIN_PAGE_OUT2				8
#define MENU_MAIN_PAGE_OUT3				9
#define MENU_MAIN_PAGE_OUT4				10

#define MENU_MAIN_PAGE_BUZZ				11
#define MENU_MAIN_PAGE_REPLAY			12


//--------------------------------------------------------
struct {
	uint8_t index;
	uint8_t edit;
	uint8_t nameEdit;
	uint8_t refresh;
	uint16_t idle;
	uint8_t mainWindow;
	uint16_t mainDelay;
} menu;

//--------------------------------------------------------
const uint8_t windows_table[] = {
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_T1,
	MENU_MAIN_PAGE_MAX1,
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_T2,
	MENU_MAIN_PAGE_MAX2,
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_TR,
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_OUT1,
	MENU_MAIN_PAGE_OUT2,
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_OUT3,
	MENU_MAIN_PAGE_OUT4,								
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_ALL,
	MENU_MAIN_PAGE_PROFILE,
	MENU_MAIN_PAGE_BUZZ
};

//----------------------------------------------------------
void MenuInit(void){

	menu.mainDelay = (MENU_MAIN_SWITCH_SPEED * 2);

}

//----------------------------------------------------------
void MenuMan(uint8_t action){
	uint8_t* trg;
	//MENU_PAGE_STRUCT *menu_item;
	int16_t action_val = 0;
	
	if (action > 0) {
		//menu.refresh = 1;
		menu.idle = MENU_IDLE_TIME;
	}
	
	//------------------------------------------------------
	//only move, not edit
	//------------------------------------------------------
	if (menu.edit == MENU_EDIT_DISABLE){
		menu.nameEdit = 0;	
		switch (action){
			case MENU_ENTER_BUTTON:{
				switch (MenuParamTypeGet(menu.index)){
					case TYPE_MAIN:{
						menu.mainWindow++;
						menu.mainDelay = (MENU_MAIN_SWITCH_SPEED);
					} break;
					
					case TYPE_PROF_PAGE:
					case TYPE_OUTPUT_PAGE:
					case TYPE_BUZZER_PAGE:
					case TYPE_SETTINGS_PAGE:{
						menu.index++;
					} break;
				
					case TYPE_PROF_SET_PAGE:
					case TYPE_OUT_STATE_PAGE:
					case TYPE_OUT_SRC_PAGE:
					case TYPE_OUT_PARAM_PAGE:
					case TYPE_BUZZ_SRC_PAGE:
					case TYPE_BUZZ_PARAM_PAGE:
					case TYPE_BUZZ_VDET_PAGE:
					case TYPE_SETT_DELTA_PAGE:{
						MenuEditModeChange(MENU_EDIT_ENABLE);
					} break;
					
					case TYPE_PROF_NAME_PAGE:
					case TYPE_OUT_NAME_PAGE:
					case TYPE_SETT_NAME_PAGE:{
						MenuEditModeChange(MENU_EDIT_ENABLE);
						menu.nameEdit = 1;						
					} break;			
				}
			} break;
			case MENU_ESC_BUTTON:{
				if (menu.index > 0){
					menu.index = pgm_read_byte(&menu_item_map[menu.index]->base);
				} else {
					menu.mainDelay = (MENU_MAIN_SWITCH_SPEED);
					menu.mainWindow = 0;					
				}
			} break;
			case MENU_UP_BUTTON:{
				menu.index = pgm_read_byte(&menu_item_map[menu.index]->next);
			} break;
			case MENU_DN_BUTTON:{
				menu.index = pgm_read_byte(&menu_item_map[menu.index]->prev);
			} break;
			case MENU_ESC_BUTTON | BUTTON_HOLD:{
				DsTempExtremeClear(0);
				DsTempExtremeClear(1);
				LCDInit();
			} break;

		}
		//------------------------------------------------------
		//only edit, not move
		//------------------------------------------------------
	} else {
		switch (action){
			case MENU_UP_BUTTON:{
				action_val = MENU_ACTION_CLICK_CHANGE;
			} break;
			case MENU_DN_BUTTON:{
				action_val = -MENU_ACTION_CLICK_CHANGE;
			} break;
			case (MENU_UP_BUTTON | MENU_REPEAT_BUTTON):{
				action_val = MENU_ACTION_REPEAT_CHANGE;
			} break;
			case (MENU_DN_BUTTON | MENU_REPEAT_BUTTON):{
				action_val = -MENU_ACTION_REPEAT_CHANGE;
			} break;
		
			case MENU_ENTER_BUTTON:{
				if (menu.nameEdit > 0){
					menu.nameEdit++;
					if (menu.nameEdit > SETT_NAME_LEN){
						menu.nameEdit = 1;
					}
				} else {
					MenuEditModeChange(MENU_EDIT_DISABLE);
				}
			} break;
			
			case MENU_ESC_BUTTON:{	
				menu.nameEdit = 0;
				MenuEditModeChange(MENU_EDIT_DISABLE);
			} break;
		}
		
		if (action_val != 0){
			switch (MenuParamTypeGet(menu.index)){	
				case TYPE_PROF_SET_PAGE:
				case TYPE_OUT_SRC_PAGE:
				case TYPE_BUZZ_SRC_PAGE:
				case TYPE_BUZZ_VDET_PAGE:
				case TYPE_SETT_DELTA_PAGE: {
					SettValueChange(action_val, menu.index);
				} break;
				
				case TYPE_OUT_STATE_PAGE:{
					SettValueChangeNoSave(action_val, menu.index);				
				} break;

				case TYPE_OUT_PARAM_PAGE:				
				case TYPE_BUZZ_PARAM_PAGE:{
					SettValueChange(action_val, menu.index);
					SettParamCorrect(action_val, menu.index);					
				} break;
				
				case TYPE_PROF_NAME_PAGE:
				case TYPE_OUT_NAME_PAGE:
				case TYPE_SETT_NAME_PAGE:{
					if (menu.nameEdit > 0){
						SettNameChange(action_val, menu.index, (menu.nameEdit - 1));					
					}
				} break;
			}		
		}
	}	
}

//----------------------------------------------------------
void MenuEditModeChange(uint8_t action){
	if (action > MENU_EDIT_CHANGE) {
		action = MENU_EDIT_DISABLE;
	}

	if (action == MENU_EDIT_CHANGE){
		if (menu.edit == MENU_EDIT_DISABLE){
			menu.edit = MENU_EDIT_ENABLE;
		} else {
			menu.edit = MENU_EDIT_DISABLE;
		}
	} else {
		menu.edit = action;
	}
}

//----------------------------------------------------------
uint8_t MenuParamTypeGet(uint8_t index){
	return (uint8_t)(pgm_read_byte(&menu_item_map[menu.index]->type));
}

//----------------------------------------------------------
void MenuRefresh(void){
	menu.refresh = 1;
}

//----------------------------------------------------------
void MenuExe(void){
	if (menu.index == MENU_INDEX_MAIN){
		if (menu.mainDelay > 0) {
			if (DispMessageCheck() == 0) menu.mainDelay--;
		}
			
		if (menu.mainDelay == 0){
			menu.mainDelay = (MENU_MAIN_SWITCH_SPEED);
			menu.mainWindow++;
			menu.refresh = 1;
		}

	} else {
		menu.mainDelay = (MENU_MAIN_SWITCH_SPEED);
		menu.mainWindow = 0;
		if (menu.idle > 0){
			menu.idle--;
			if (menu.idle == 0){
				menu.index = 0;
				menu.edit = MENU_EDIT_DISABLE;
				menu.refresh = 1;
			}
		}
	}
}

//----------------------------------------------------------
void MenuProcess(){
	if (menu.refresh > 0){
		menu.refresh = 0;
		if (menu.index > MENU_INDEX_MAIN){
			MenuView(menu.index);
		}else{
			MenuMainWindowView();
		}
	}
}

//----------------------------------------------------------
uint8_t MenuIndexGet(void){
	return menu.index;
}

//----------------------------------------------------------
void MenuMainWindowView(void){
	uint8_t window;
	
	window = windows_table[menu.mainWindow];
	if (window >= MENU_MAIN_PAGE_REPLAY){
		menu.mainWindow = 0;
		window = MENU_MAIN_PAGE_ALL;
	}
	
	DispBuffClear();
	DispBuffSetPos(0,0);

	switch (window){
		case MENU_MAIN_PAGE_ALL:
			MenuDispChannelTemp(SETT_SRC_T1); DispBuffSetPos(0,9); MenuDispChannelTemp(SETT_SRC_T2);
			DispBuffSetPos(1,3); DispBuffAddText("Tr ="); MenuDispTemp(TempGet(SETT_SRC_DELTA));
		break;
		case MENU_MAIN_PAGE_T1:
			DispBuffAddText("  T1 = "); MenuDispChannelTemp(SETT_SRC_T1); 
			DispBuffSetPos(1,2); MenuDispNme(SettValueSrc(MENU_INDEX_SETT_NAME1));
		break;
		case MENU_MAIN_PAGE_T2:
			DispBuffAddText("  T2 = "); MenuDispChannelTemp(SETT_SRC_T2);
			DispBuffSetPos(1,2); MenuDispNme(SettValueSrc(MENU_INDEX_SETT_NAME2));
		break;
		case MENU_MAIN_PAGE_TR:
			DispBuffAddText("  Tr = "); MenuDispTemp(TempGet(SETT_SRC_DELTA));
			DispBuffSetPos(1,2); DispBuffAddText("Roznica temp  ");
		break;
		case MENU_MAIN_PAGE_MAX1:
			DispBuffAddText("Wart.skrajne T1 ");
			DispBuffAddByte(LCD_SPEC_CHAR_MAX); MenuDispTemp(DsTempExtremeGet(0,1));
			DispBuffSetPos(1,9);
			DispBuffAddByte(LCD_SPEC_CHAR_MIN); MenuDispTemp(DsTempExtremeGet(0,0));			
		break;
		case MENU_MAIN_PAGE_MAX2:
			DispBuffAddText("Wart.skrajne T2 ");
			DispBuffAddByte(LCD_SPEC_CHAR_MAX); MenuDispTemp(DsTempExtremeGet(1,1));
			DispBuffSetPos(1,9);
			DispBuffAddByte(LCD_SPEC_CHAR_MIN); MenuDispTemp(DsTempExtremeGet(1,0));
		break;
		/*case MENU_MAIN_PAGE_MAXR:
			DispBuffAddText("Wart.skrajne Tr ");
			DispBuffAddByte(LCD_SPEC_CHAR_MAX); MenuDispTemp(1000);
			DispBuffSetPos(1,9);
			DispBuffAddByte(LCD_SPEC_CHAR_MIN); MenuDispTemp(-107);
		break;*/
		case MENU_MAIN_PAGE_OUT1:
			DispBuffAddText(" Wyjscie 1 "); DispBuffAddPgmText(menu_state_map[(*SettValueSrc(MENU_INDEX_OUTPUT1_STATE))]);
			DispBuffSetPos(1,2); MenuDispNme(SettValueSrc(MENU_INDEX_OUTPUT1_NAME));
		break;
		case MENU_MAIN_PAGE_OUT2:
			DispBuffAddText(" Wyjscie 2 "); DispBuffAddPgmText(menu_state_map[(*SettValueSrc(MENU_INDEX_OUTPUT2_STATE))]);
			DispBuffSetPos(1,2); MenuDispNme(SettValueSrc(MENU_INDEX_OUTPUT2_NAME));
		break;
		case MENU_MAIN_PAGE_OUT3:
			DispBuffAddText(" Wyjscie 3 "); DispBuffAddPgmText(menu_state_map[(*SettValueSrc(MENU_INDEX_OUTPUT3_STATE))]);
			DispBuffSetPos(1,2); MenuDispNme(SettValueSrc(MENU_INDEX_OUTPUT3_NAME));
		break;
		case MENU_MAIN_PAGE_OUT4:
			DispBuffAddText(" Wyjscie 4 "); DispBuffAddPgmText(menu_state_map[(*SettValueSrc(MENU_INDEX_OUTPUT4_STATE))]);
			DispBuffSetPos(1,2); MenuDispNme(SettValueSrc(MENU_INDEX_OUTPUT4_NAME));
		break;		
		case MENU_MAIN_PAGE_PROFILE:
			DispBuffAddText(" Aktywny profil ");
			DispBuffSetPos(1,2); 
			uint8_t profile = *SettValueSrc(MENU_INDEX_PROFILE_SET);
			MenuDispNme(SettValueSrc(MENU_INDEX_PROFILE_NAME1 + (profile * SETT_NAME_LEN)));
		break;
		case MENU_MAIN_PAGE_BUZZ:
			DispBuffAddText(" Temp. alarmow  ");
			DispBuffSetPos(1,0);
			DispBuffAddByte(LCD_SPEC_CHAR_MAX); MenuDispTemp(*SettValueSrc(MENU_INDEX_BUZZER_COND1P));
			DispBuffSetPos(1,9);
			DispBuffAddByte(LCD_SPEC_CHAR_MIN); MenuDispTemp(*SettValueSrc(MENU_INDEX_BUZZER_COND2P));
		break;
	}
}

//----------------------------------------------------------
void MenuView(int16_t menu_index){
	uint8_t i = 0;
		
	DispBuffClear();
	DispBuffSetPos(0,0);
	
	if (menu.nameEdit == 0){
		DispBuffAddPgmText(pgm_read_ptr(&menu_item_map[menu_index]->name));
	} else {
		DispBuffSetPos(0, (menu.nameEdit + 1));
		DispBuffAddByte('?');		
	}


	DispBuffSetPos(1,2);
	
	switch (MenuParamTypeGet(menu_index)){
		
		case TYPE_PROF_PAGE:
		case TYPE_PROF_SET_PAGE:{
			i = *SettValueSrc(menu_index);
			MenuDispNme(SettValueSrc(MENU_INDEX_PROFILE_NAME1 + i));
		}break;
		
		case TYPE_BUZZ_VDET_PAGE:
		case TYPE_OUT_STATE_PAGE:{
			DispBuffAddPgmText(menu_state_map[(*SettValueSrc(menu_index))]);
		} break;
		
		case TYPE_OUT_SRC_PAGE:
		case TYPE_BUZZ_SRC_PAGE:{
			DispBuffAddPgmText(menu_src_map[(*SettValueSrc(menu_index))]);
		} break;
		
		case TYPE_OUT_PARAM_PAGE:
		case TYPE_BUZZ_PARAM_PAGE:{
			MenuDispTemp(*SettValueSrc(menu_index));
		} break;
		
		case TYPE_PROF_NAME_PAGE:
		case TYPE_OUT_NAME_PAGE:
		case TYPE_SETT_NAME_PAGE:{
			MenuDispNme(SettValueSrc(menu_index));
		} break;

		case TYPE_SETT_DELTA_PAGE:{
			DispBuffAddText("Tr=");
			DispBuffAddPgmText(menu_delta_map[(*SettValueSrc(menu_index))]);
		}break;
	}
	
	if (menu.edit == MENU_EDIT_ENABLE){
		DispBuffAddPosText(MENU_EDIT_CHARACTER1, 1, 0);
		DispBuffAddPosText(MENU_EDIT_CHARACTER2, 1, 15);
	}
}

//---------------------------------------------------------------------------------
void MenuDispTemp(int16_t val){
	uint8_t visible = 0;
	uint8_t d;
	uint8_t status = 0;
	uint16_t div = 1000;


	//if ((val >= -550) || (val <= 1250)){
		if (val < -999) val = -999;
		if (val < 0) {
			DispBuffAddByte('-');
			val *= -1;
			div /= 10;
		}

		//val /= 10;		//cut decimal

		while (status == 0)
		{
			d = (val / div);
			if (d) visible = 1;
			if (div == 10) visible = 1;

			if (visible) {
				d = (d + '0');
			} else {
				d = (' ');
			}

			DispBuffAddByte(d);
			if (div == 10)
				DispBuffAddByte('.');

			if (div == 1) status = 1;

			val = (val % div);
			div = (div / 10);
		}
		
		DispBuffAddByte(LCD_SPEC_CHAR_DEGREE);
	//}
}

//---------------------------------------------------------------------------------
void MenuDispChannelTemp(uint8_t ch){
	if (DsStatusGet(ch - 1) >= DS_NO_CONNECT){
		DispBuffAddText(" ---- ");
	} else {
		MenuDispTemp(TempGet(ch));
	}
}

//---------------------------------------------------------------------------------
void MenuDispNme(uint8_t *src){
	DispBuffAdd(src, 0, SETT_NAME_LEN);
}
