/*
 * settings.c
 *
 * Created: 25.09.2017 12:33:35
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <avr/pgmspace.h>

#include "defs.h"
#include "menu.h"
#include "menu_defs.h"
#include "settings.h"


#define SETT_EEMEM_OFFSET			0
#define SETT_SAVE_DELAY				3

#define SETT_ARRAY_SIZE				SETTI_INDEX_END	
uint8_t sett_array[SETT_ARRAY_SIZE];

struct{
	int16_t needSaveCounter;
}sett;


//----------------------------------------------------------
void SettInit(void){
	
	SettLoad();
		
	SettValueCorrect(MENU_INDEX_PROFILE_SET);
	SettValueCorrect(MENU_INDEX_SETT_DELTA);
	
	SettInitOutputParameters();
	SettInitBuzzerParameters();
	SettInitSensorsParameters();
	
	SettInitNames(MENU_INDEX_PROFILE_NAME1, menu_prof_map[0]);
	SettInitNames(MENU_INDEX_PROFILE_NAME2, menu_prof_map[1]);
	SettInitNames(MENU_INDEX_PROFILE_NAME3, menu_prof_map[2]);
	
	SettInitNames(MENU_INDEX_OUTPUT1_NAME, menu_out_map[0]);
	SettInitNames(MENU_INDEX_OUTPUT2_NAME, menu_out_map[1]);
	SettInitNames(MENU_INDEX_OUTPUT3_NAME, menu_out_map[2]);
	SettInitNames(MENU_INDEX_OUTPUT4_NAME, menu_out_map[3]);
	
	SettInitNames(MENU_INDEX_SETT_NAME1, menu_sensor_map[0]);
	SettInitNames(MENU_INDEX_SETT_NAME2, menu_sensor_map[1]);
	
	/**SettValueSrc(MENU_INDEX_OUTPUT1_STATE) = OUT_STATE_OFF;
	*SettValueSrc(MENU_INDEX_OUTPUT2_STATE) = OUT_STATE_OFF;
	*SettValueSrc(MENU_INDEX_OUTPUT3_STATE) = OUT_STATE_OFF;
	*SettValueSrc(MENU_INDEX_OUTPUT4_STATE) = OUT_STATE_OFF;
	*/
}

//----------------------------------------------------------
void SettInitOutputParameters(void){
	uint8_t menu_index = 0;
	uint8_t save_profile = SettProfileGet();

	for(uint8_t i = 0; i < 4; i++){
		
		menu_index = (MENU_INDEX_OUTPUT1_COND1P + (i * MENU_OUTPUT_GROUP_LEN));
		
		for (uint8_t profile = 0; profile < SETT_PROFILE_MAN; profile++){
			SettProfileSet(profile);
			
			if ((*SettValueSrc(menu_index) == *SettValueSrc(menu_index+1)) && ((*SettValueSrc(menu_index) == 0)||(*SettValueSrc(menu_index) == -1))){
			
				*SettValueSrc(menu_index) = (OUT1_PAR1_DEFAULT + (i*100) - (profile * 50));
				*SettValueSrc(menu_index+1) = (OUT1_PAR1_DEFAULT - 50 + (i*100) - (profile * 50));
			}
		}
		
		menu_index = (MENU_INDEX_OUTPUT1_CONDSRC + (i * MENU_OUTPUT_GROUP_LEN));
				
		if (SettValueCorrect(menu_index) > 0) {
			*SettValueSrc(menu_index) = SETT_SRC_T1;
		}
	}
	SettProfileSet(save_profile);
}

//----------------------------------------------------------
void SettInitBuzzerParameters(void){
	uint8_t menu_index = MENU_INDEX_BUZZER_COND1P;
	
	if ((*SettValueSrc(menu_index) == *SettValueSrc(menu_index+1)) && ((*SettValueSrc(menu_index) == 0)||(*SettValueSrc(menu_index) == -1))){
		*SettValueSrc(menu_index) = BUZZ_PAR1_DEFAULT;
		*SettValueSrc(menu_index+1) = BUZZ_PAR2_DEFAULT;
	}
	
	menu_index = MENU_INDEX_BUZZER_CONDSRC;
	if (SettValueCorrect(menu_index) > 0) {
		*SettValueSrc(menu_index) = SETT_SRC_T1;
	}
	
	menu_index = MENU_INDEX_BUZZER_VDET;
	if (SettValueCorrect(menu_index) > 0) {
		*SettValueSrc(menu_index) = 0;
	}
}

//----------------------------------------------------------
void SettInitSensorsParameters(void){
	if (SettArrayGetVal16(MENU_INDEX_SETT_DELTA) > SETT_DELTA_MAX){
		SettArraySetVal16(MENU_INDEX_SETT_DELTA, SETT_DELTA_MIN);
	}	
}

//----------------------------------------------------------
void SettInitNames(uint8_t menu_index, PGM_P *src){
	if (SettNameCheck(SettValueSrc(menu_index)) > 0){
		SettNameSetPgm(src, menu_index);
	}
}

//----------------------------------------------------------
//----------------------------------------------------------
uint8_t SettArrayIndexGet(uint8_t menu_index){
	return (uint8_t)(pgm_read_byte(&menu_item_map[menu_index]->setti));
}
//----------------------------------------------------------
uint8_t SettArrayTypeGet(uint8_t menu_index){
	return (uint8_t)(pgm_read_byte(&menu_item_map[menu_index]->type));	
}
//----------------------------------------------------------
int16_t SettArrayMinGet(uint8_t menu_index){
	return (uint16_t)(pgm_read_word(&menu_item_map[menu_index]->min));
}
//----------------------------------------------------------
int16_t SettArrayMaxGet(uint8_t menu_index){
	return (uint16_t)(pgm_read_word(&menu_item_map[menu_index]->max));
}

//----------------------------------------------------------
//----------------------------------------------------------
void SettArraySetVal16(int16_t val, uint8_t array_index){
	sett_array[array_index] = (0xFF & val);
	sett_array[array_index + 1] = (0xFF & (val >> 8));
}

//----------------------------------------------------------
int16_t SettArrayGetVal16(uint8_t array_index){
	int16_t val = 0;
	val = sett_array[array_index];
	val += ((sett_array[array_index + 1]) << 8);
	return val;
}

//----------------------------------------------------------
//----------------------------------------------------------
uint8_t SettProfileGet(void){
	if (SettArrayGetVal16(SETTI_INDEX_PROFILES_SET) > SETT_PROFILE_MAX){
		SettArraySetVal16(SETT_PROFILE1, SETTI_INDEX_PROFILES_SET);
	}
	return SettArrayGetVal16(SETTI_INDEX_PROFILES_SET);
}

//----------------------------------------------------------
void SettProfileSet(uint8_t new_val){
	if (new_val > SETT_PROFILE_MAX) new_val = SETT_PROFILE1;
	SettArraySetVal16(new_val, SETTI_INDEX_PROFILES_SET);
}

//----------------------------------------------------------
int16_t *SettValueSrc(uint8_t menu_index){
	static int16_t val = 0;
	int16_t* val_src = 0;
	uint8_t ai = 0;
	
	ai = SettArrayIndexGet(menu_index);
	val_src = &sett_array[ai];
	
	if (SettProfileGet() > 0){
		if ((SettProfileGet() > 0) && (SettProfileGet() < SETT_PROFILE_MAN)){	
			for (uint8_t i = 0; i<4; i++){
				
				if ((menu_index == (MENU_INDEX_OUTPUT1_COND1P + (i*MENU_OUTPUT_GROUP_LEN)) ) ||
					(menu_index == (MENU_INDEX_OUTPUT1_COND2P + (i*MENU_OUTPUT_GROUP_LEN)) ) ) {

					val_src = &(sett_array[ai + 4]);
				}
			}
		}

		if ((SettProfileGet() == SETT_PROFILE_MAN)){
			for (uint8_t i = 0; i<4; i++){
				
				if ((menu_index == (MENU_INDEX_OUTPUT1_COND1P + (i*MENU_OUTPUT_GROUP_LEN)) ) ||
					(menu_index == (MENU_INDEX_OUTPUT1_COND2P + (i*MENU_OUTPUT_GROUP_LEN)) ) ) {
					val = SETT_TEMP_PAR_MAX;
					val_src = &val;
				}
				
				if (menu_index == (MENU_INDEX_OUTPUT1_CONDSRC + (i*MENU_OUTPUT_GROUP_LEN))){
					val = SETT_SRC_MAN;
					val_src = &val;					
				}
			}		
		}
	}
	
	return val_src;
}

//----------------------------------------------------------
uint8_t SettNameCheck(uint8_t *src){
	for(uint8_t i = 0; i < SETT_NAME_LEN; i++){
		if ((*src < ' ') || (*src > 'z')){
			return 1;
		}
		src++;
	}
	return 0;
}

//----------------------------------------------------------
void SettNameSetPgm(uint8_t *src, uint8_t menu_index){
	uint8_t *dst;
	dst = SettValueSrc(menu_index);
	
	for (uint8_t i = 0; i < SETT_NAME_LEN; i++){
		if ((pgm_read_byte(src) > 0) && (src > 0)){
			*dst = (uint8_t)pgm_read_byte(src);
			src++;
		} else {
			*dst = ' ';
		}
		dst++;
	}
}

//----------------------------------------------------------
uint8_t SettValueCorrect(uint8_t menu_index){
	int16_t* check_val = SettValueSrc(menu_index);
	
	uint8_t result = 0;
	if (*check_val < SettArrayMinGet(menu_index)){
		*check_val = SettArrayMinGet(menu_index);
		result = 1;
	}
	if (*check_val > SettArrayMaxGet(menu_index)){
		*check_val = SettArrayMaxGet(menu_index);
		result = 2;
	}
	return result;
}

//----------------------------------------------------------
uint8_t SettValueChange(int16_t action, uint8_t menu_index){
	*SettValueSrc(menu_index) += action;
	SettValueCorrect(menu_index);
	sett.needSaveCounter = ((SETT_SAVE_DELAY * 1000) / MAIN_DELAY);
}

//----------------------------------------------------------
uint8_t SettValueChangeNoSave(int16_t action, uint8_t menu_index){
	*SettValueSrc(menu_index) += action;
	SettValueCorrect(menu_index);
}

//----------------------------------------------------------
uint8_t SettParamCorrect(uint8_t action_val, uint8_t menu_index){
	
	for (uint8_t i = 0; i<4; i++){
				
		if (menu_index == (MENU_INDEX_OUTPUT1_COND1P + (i*MENU_OUTPUT_GROUP_LEN))){

			if (*SettValueSrc(menu_index) == *SettValueSrc(menu_index + 1)){				
				//if ((*SettValueSrc(menu_index) > SETT_TEMP_PAR_MIN) && (*SettValueSrc(menu_index) < SETT_TEMP_PAR_MAX)){
					if (action_val >= 0) {
						(*SettValueSrc(menu_index))++;
					} else {
						(*SettValueSrc(menu_index))--;
					}					
				//}
			}			
		}
		
		if (menu_index == (MENU_INDEX_OUTPUT1_COND2P + (i*MENU_OUTPUT_GROUP_LEN))){
			if (*SettValueSrc(menu_index) == *SettValueSrc(menu_index - 1)){
				//if ((*SettValueSrc(menu_index) > SETT_TEMP_PAR_MIN) && (*SettValueSrc(menu_index) < SETT_TEMP_PAR_MAX)){
					if (action_val >= 0) {
						(*SettValueSrc(menu_index))++;
						} else {
						(*SettValueSrc(menu_index))--;
					}
				//}
			}			
		}
	}	
}

//----------------------------------------------------------
uint8_t SettNameChange(int16_t action, uint8_t menu_index, uint8_t edit_index){
	uint8_t* value_src;
	value_src = SettValueSrc(menu_index);
	value_src += edit_index;
	*value_src += action;
	
	if (*value_src < SettArrayMinGet(menu_index)){
		*value_src = SettArrayMinGet(menu_index);
	}
	if (*value_src > SettArrayMaxGet(menu_index)){
		*value_src = SettArrayMaxGet(menu_index);
	}
	sett.needSaveCounter = ((SETT_SAVE_DELAY * 1000) / MAIN_DELAY);
}

//---------------------------------------------------------
void SettProc(void){
	uint8_t menu_index = MenuIndexGet();
	if (menu_index == MENU_INDEX_MAIN){
		if (sett.needSaveCounter > 0){
			sett.needSaveCounter--;
			if (sett.needSaveCounter == 0){		
				SettSave();
				DispNewMessage("  Ustawienia       zapamietane", 3);
			}
		}
	}
}

//---------------------------------------------------------
void SettLoad(void){
	EERestore(&sett_array[0],  SETT_EEMEM_OFFSET, SETT_ARRAY_SIZE);
}

//---------------------------------------------------------
void SettSave(void){
	cli();
	EEStore(&sett_array[0],  SETT_EEMEM_OFFSET, SETT_ARRAY_SIZE);
	sei();
}

//---------------------------------
static void EEStore(uint8_t *pt, uint8_t adr, uint8_t len){
	while (len)
	{
		EEwrite(adr, *pt);
		adr++;
		pt++;
		len--;
	}
}

//---------------------------------
static void EERestore(uint8_t *pt, uint8_t adr, uint8_t len){
	while (len)
	{
		*pt = EEread(adr);
		adr++;
		pt++;
		len--;
	}
}

//---------------------------------
static void EEwrite(uint16_t address, uint8_t data){
	while(EECR & (1<<EEPE)); 		//Wait for completion of previous write
	EECR = 0;						//Set Programming mode

	EEAR = address;					//Set up address and data registers
	EEDR = data;

	EECR |= (1<<EEMPE);				//Write logical one to EEMPE
	EECR |= (1<<EEPE);				//Start eeprom write by setting EEPE
}

//---------------------------------
static uint8_t EEread(uint16_t address){
	while(EECR & (1<<EEPE));		//Wait for completion of previous write
	
	EEAR = address;					//Set up address register
	EECR |= (1<<EERE);				//Start eeprom read by writing EERE
	
	return EEDR;					//Return data from data register
}